/*   
  *   *  
 *  Copyright (C) 2001-2005 Edscott Wilson Garcia under GNU GPL
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
static void *smb_object;

static xfdir_t smb_xfdir;
static GList *listSMB=NULL;
static GList *listSHARES=NULL;
static GList *listMASTERS=NULL;
static int   smb_count;
static char *smb_server;
static char *smb_pass;
static gboolean samba_server;
static int cual_chingao,query_result=UNDEFINED;

typedef struct share_t{
	char *share;
	unsigned type;
}share_t;

#include "smblookup.i"

static void 
free_share_t(gpointer data,gpointer user_data){
	share_t *d_share=(share_t *)data;
	g_free(d_share->share);
	d_share->share=NULL;
	g_free(d_share);
	d_share=NULL;
}
	


static
int smb_stderr(int n, void *data, void *user_data)
{
    widgets_t *widgets_p=user_data;
    char *line;
    if(n)
	return TRUE;		/* this would mean binary data */
    line = (char *)data;
	print_diagnostics(widgets_p,NULL, line, NULL);
    return TRUE;
}


static
void 
free_data(gpointer data,gpointer user_data){
	g_free((char *)(data));
	data=NULL;
}


static void
printout_listSMB(gpointer data,gpointer user_data){
  char *netbios=(char *)data;
		  
		  
  smb_xfdir.gl[smb_count].pathv=g_strdup(netbios);
  
  smb_xfdir.gl[smb_count].en=mk_entry(0);
  SET_NETWORK_TYPE(smb_xfdir.gl[smb_count].en->type);
  SET_XF_NETWS(smb_xfdir.gl[smb_count].en->subtype);
  SET_XF_NODE(smb_xfdir.gl[smb_count].en->type);
  if (samba_server) SET_SAMBA_SERVER(smb_xfdir.gl[smb_count].en->subtype);
  smb_xfdir.gl[smb_count].en->path=
	(char *)malloc(strlen(netbios)+strlen("smb:///")+1);
  sprintf(smb_xfdir.gl[smb_count].en->path,"smb://%s/",netbios);
  smb_xfdir.gl[smb_count].en->st=(struct stat *)malloc(sizeof(struct stat));
  smb_xfdir.gl[smb_count].en->st->st_size=0;
  smb_xfdir.gl[smb_count].en->st->st_mtime=time(NULL);
  smb_xfdir.gl[smb_count].en->st->st_ctime=time(NULL);
  smb_xfdir.gl[smb_count].en->st->st_gid=(gid_t)-1;
  smb_xfdir.gl[smb_count].en->st->st_uid=(uid_t)-1;
  smb_xfdir.gl[smb_count].en->st->st_mode=S_IFBLK;
  if (smb_pass) smb_xfdir.gl[smb_count].en->tag=g_strdup(smb_pass);
  else {
	  if (getenv("SMB_USER") && strlen(getenv("SMB_USER"))
			  && strchr(getenv("SMB_USER"),'%'))
		  smb_xfdir.gl[smb_count].en->tag=g_strdup(getenv("SMB_USER"));
	  else 
		  smb_xfdir.gl[smb_count].en->tag=g_strdup("GUEST%%");
  }
  SET_XF_NETWS(smb_xfdir.gl[smb_count].en->subtype);
  smb_count++;
}


static void
SMBForkOver (pid_t pid, void *user_data)
{	
  smb_object = NULL;
  return;
}


static
xfdir_t *
private_get_xfdir(widgets_t *widgets_p, record_entry_t *en){
  char *argument[7];
  int i;
  gchar *servidor;
  
  if (!en||!en->path){
      return NULL;
  }
  if (strncmp(en->path,"smb:",strlen("smb:"))==0) servidor = en->path+strlen("smb:");
  else if (strncmp(en->path,"SMB:",strlen("SMB:"))==0) servidor = en->path+strlen("SMB:");
  else servidor=en->path;

  
  if (!servidor || !strlen(servidor) || smb_object){
     return NULL;
  }
  smb_server=servidor; 
  samba_server=FALSE;
  cual_chingao = 0;
  {
    gchar *g=g_strdup_printf(_("Querying %s"), servidor);
    print_status(widgets_p,NULL,g,NULL);
    g_free(g);
  }

  /* bla bla section */
  print_diagnostics(widgets_p,NULL,"XFSAMBA> smbclient -N -L ",
		  servidor,"\n",NULL);
  
   if (listSMB){
	g_list_foreach(listSMB,free_data,NULL);
      	g_list_free(listSMB);
	listSMB=NULL;
  }
  if (strncmp(servidor,"//",strlen("//"))==0) listSMB = g_list_append (listSMB,(gpointer)(g_strdup(servidor+2)));
  
  if (listSHARES){
	g_list_foreach(listSHARES,free_share_t,NULL);
      	g_list_free(listSHARES);
	listSHARES=NULL;
  }
  if (listMASTERS){
	g_list_foreach(listMASTERS,free_data,NULL);
      	g_list_free(listMASTERS);
	listMASTERS=NULL;
  }

  i=0;
  argument[i++]=  "smbclient";
  argument[i++]=  "-N";
  argument[i++]=  "-L";
  argument[i++]=  servidor;
  argument[i++]=  0;

  
  query_result=UNDEFINED;
    show_text(widgets_p);
  smb_object = Tubo_full (fork_function, 
	(void *)argument, 
	SMBForkOver, 
	NULL, 
	SMBparseLookup, 
	smb_stderr,widgets_p,15);
  while (smb_object){
	 set_progress_generic(widgets_p,-1,-1,1);
     while (gtk_events_pending()) gtk_main_iteration();
     usleep(5000);
  }
    hide_text(widgets_p->diagnostics);
  if (listSMB){
  	print_status(widgets_p,"xffm/info", _("Query done"),NULL);
        smb_xfdir.pathc = g_list_length(listSMB);
        smb_xfdir.gl = (dir_t *) malloc(smb_xfdir.pathc * sizeof(dir_t));
	smb_count=0;
	g_list_foreach(listSMB,printout_listSMB,NULL);
  } else { 
	  print_status(widgets_p,"xffm/info",_("Query failed"),NULL);
	  smb_xfdir.pathc = 0;
	  return NULL;
  }
  return &smb_xfdir;
}


